/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.sql.fn;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;

import woolpack.fn.Fn;
import woolpack.fn.FnUtils;

/**
 * 委譲先からクエリを取得して
 * {@link Statement#execute(String)}を実行し、
 * 返却値の生成を委譲する{@link Fn}です。
 * @author nakamura
 *
 * @param <C>
 */
public class StatementInput<C, R> implements Fn<C, R> {
	private DataSource dataSource;
	private Fn<? super C, ? extends String> queryFactory;
	private Fn<? super Statement, ? extends R> converter;
	
	/**
	 * 
	 * @param dataSource データソース。
	 * @param queryFactory クエリ生成の委譲先。
	 */
	public StatementInput(
			final DataSource dataSource,
			final Fn<? super C, ? extends String> queryFactory,
			final Fn<? super Statement, ? extends R> converter) {
		this.dataSource = dataSource;
		this.queryFactory = queryFactory;
		this.converter = converter;
	}
	
	/**
	 * 
	 * @param dataSource データソース。
	 * @param query クエリ。
	 */
	public StatementInput(
			final DataSource dataSource,
			final String query,
			final Fn<? super Statement, ? extends R> converter) {
		this(dataSource, FnUtils.fix(query), converter);
	}

	public R exec(final C c) {
		try {
			final Connection connection = dataSource.getConnection();
			try {
				final Statement statement = connection.createStatement();
				try {
					statement.execute(queryFactory.exec(c));
					return converter.exec(statement);
				} finally {
					statement.close();
				}
			} finally {
				connection.close();
			}
		} catch (final SQLException e) {
			throw new IllegalStateException(e);
		}
	}

	public DataSource getDataSource() {
		return dataSource;
	}
	public void setDataSource(final DataSource dataSource) {
		this.dataSource = dataSource;
	}
	public Fn<? super C, ? extends String> getQueryFactory() {
		return queryFactory;
	}
	public void setQueryFactory(final Fn<? super C, ? extends String> queryFactory) {
		this.queryFactory = queryFactory;
	}
	public Fn<? super Statement, ? extends R> getConverter() {
		return converter;
	}
	public void setConverter(final Fn<? super Statement, ? extends R> converter) {
		this.converter = converter;
	}
}
